/*
 * flood.h  塗りつぶし操作 
 *
 * Copyright (C) 1998- 田尻さん <tajiri@wizard.elec.waseda.ac.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/
/* $Id: flood.h,v 1.3 2006/04/21 16:40:48 chikama Exp $ */

#ifndef __FLOOD_H__
#define __FLOOD_H__

int flood(int seedx, int seedy, int(*vist)(int, int));

#endif /* __FLOOD_H__ */


