/*
 * audio_dmy.c  dummy lowlevel acess
 *
 * Copyright (C) 1997-1998 Masaki Chikama (Wren) <chikama@kasumi.ipl.mech.nagoya-u.ac.jp>
 *               1998-                           <masaki-c@is.aist-nara.ac.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/
/* $Id: audio_dmy.c,v 1.3 2000/07/27 14:55:06 chikama Exp $ */

#include "config.h"
#include "audio.h"

audio_init_t dmy_initilizer = {
	'd',
	audio_dmy_new,
	audio_dmy_free,
	mixer_dmy_new,
	mixer_dmy_free
};

audio_pcm_t *audio_dmy_new(audio_t *audio) {
	return NULL;
}

void audio_dmy_free(audio_pcm_t *pcm) {
}

