/*
 * audio_esd.h  esd lowlevel acess
 *
 * Copyright (C) 1999-2000 <tajiri@wizard.elec.waseda.ac.jp>
 * rewrited      2000-     Fumihiko Murata <fmurata@p1.tcnet.ne.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/
/* $Id: audio_esd.h,v 1.4 2002/08/18 09:35:29 chikama Exp $ */

#ifndef __AUDIO_ESD__
#define __AUDIO_ESD__

#include <esd.h>

typedef struct {
	int fdh;  /* socket to host */
	int fds;  /* stream socket discpriter */
	int sid;  /* stream source id */
	int curvol; /* current volume setting */
} audio_esd_t;

#define ESDNAME_BASE "xsystem35"

#endif /* !__AUDIO_ESD__ */
