int cp932tosjis0213(int code) {
	int ret;
	
	switch(code){
	case 0x8160:	ret = 0x81b0; break;
	case 0x8161:	ret = 0x81d2; break;
	case 0x817c:	ret = 0x81af; break;
	case 0x8790:	ret = 0x81e0; break;
	case 0x8791:	ret = 0x81df; break;
	case 0x8792:	ret = 0x81e7; break;
	case 0x8795:	ret = 0x81e3; break;
	case 0x8796:	ret = 0x81db; break;
	case 0x8797:	ret = 0x81da; break;
	case 0x879a:	ret = 0x81e6; break;
	case 0x879b:	ret = 0x81bf; break;
	case 0x879c:	ret = 0x81be; break;
	case 0xed40:	ret = 0xedb5; break;
	case 0xed41:	ret = 0xee8e; break;
	case 0xed42:	ret = 0xef58; break;
	case 0xed43:	ret = 0xef4d; break;
	case 0xed44:	ret = 0xf8d3; break;
	case 0xed45:	ret = 0x87b7; break;
	case 0xed46:	ret = 0xf580; break;
	case 0xed47:	ret = 0xeb54; break;
	case 0xed48:	ret = 0xeb89; break;
	case 0xed49:	ret = 0xfb42; break;
	case 0xed4a:	ret = 0xeb56; break;
	case 0xed4b:	ret = 0xeab8; break;
	case 0xed4c:	ret = 0x87a2; break;
	case 0xed4d:	ret = 0x87a6; break;
	case 0xed4f:	ret = 0xf05e; break;
	case 0xed50:	ret = 0x87a9; break;
	case 0xed52:	ret = 0x87ae; break;
	case 0xed53:	ret = 0xf06c; break;
	case 0xed54:	ret = 0x87b3; break;
	case 0xed55:	ret = 0xf06d; break;
	case 0xed56:	ret = 0x87b4; break;
	case 0xed58:	ret = 0x87bf; break;
	case 0xed59:	ret = 0xf07c; break;
	case 0xed5b:	ret = 0xf07b; break;
	case 0xed5c:	ret = 0xf080; break;
	case 0xed5d:	ret = 0xf087; break;
	case 0xed5e:	ret = 0xf07e; break;
	case 0xed5f:	ret = 0x87c2; break;
        case 0xed62:	ret = 0x87cd; break;
	case 0xed64:	ret = 0x87d2; break;
	case 0xed67:	ret = 0x87db; break;
	case 0xed69:	ret = 0xf162; break;
	case 0xed6b:	ret = 0x87e5; break;
	case 0xed6c:	ret = 0x87e9; break;
	case 0xed6d:	ret = 0x87ea; break;
	case 0xed6e:	ret = 0xf16d; break;
	case 0xed70:	ret = 0x87f0; break;
	case 0xed71:	ret = 0x87f2; break;
	case 0xed74:	ret = 0x87f6; break;
	case 0xed75:	ret = 0xf18c; break;
	case 0xed76:	ret = 0x8840; break;
	case 0xed77:	ret = 0x8842; break;
	case 0xed78:	ret = 0x8849; break;
	case 0xed7b:	ret = 0xf1ea; break;
	case 0xed7c:	ret = 0x886e; break;
	case 0xed7d:	ret = 0x886d; break;
	case 0xed7e:	ret = 0x886a; break;
	case 0xed80:	ret = 0x8876; break;
	case 0xed81:	ret = 0x887c; break;
	case 0xed83:	ret = 0xf259; break;
	case 0xed85:	ret = 0xf264; break;
	case 0xed86:	ret = 0x8889; break;
	case 0xed88:	ret = 0x888d; break;
	case 0xed8a:	ret = 0x9875; break;
	case 0xed8b:	ret = 0xf0a2; break;
	case 0xed8c:	ret = 0xecc7; break;
	case 0xed8d:	ret = 0x9878; break;
	case 0xed8e:	ret = 0x9879; break;
	case 0xed8f:	ret = 0x987b; break;
	case 0xed90:	ret = 0xf0c1; break;
	case 0xed91:	ret = 0x9889; break;
	case 0xed93:	ret = 0x9891; break;
	case 0xed94:	ret = 0x9895; break;
	case 0xed95:	ret = 0x9892; break;
	case 0xed96:	ret = 0xf0d3; break;
	case 0xed97:	ret = 0x9897; break;
	case 0xed98:	ret = 0x989c; break;
	case 0xed99:	ret = 0xf0e5; break;
	case 0xed9c:	ret = 0xeab4; break;
	case 0xed9d:	ret = 0xeabc; break;
	case 0xed9e:	ret = 0xeac3; break;
	case 0xed9f:	ret = 0xeac5; break;
	case 0xeda2:	ret = 0xf2cb; break;
	case 0xeda4:	ret = 0xead3; break;
	case 0xeda6:	ret = 0xf2d6; break;
	case 0xeda8:	ret = 0xead9; break;
	case 0xeda9:	ret = 0xead8; break;
	case 0xedaa:	ret = 0xf2e7; break;
	case 0xedac:	ret = 0xf341; break;
	case 0xedad:	ret = 0xeaf2; break;
	case 0xedae:	ret = 0xeaf8; break;
	case 0xedaf:	ret = 0xf36f; break;
	case 0xedb0:	ret = 0xeafa; break;
	case 0xedb2:	ret = 0xeb4b; break;
	case 0xedb3:	ret = 0xeb4d; break;
	case 0xedb5:	ret = 0xeb4c; break;
	case 0xedb7:	ret = 0xeb4e; break;
	case 0xedb8:	ret = 0xeb51; break;
	case 0xedb9:	ret = 0xeb5d; break;
	case 0xedba:	ret = 0xf39e; break;
	case 0xedbb:	ret = 0xeb5a; break;
	case 0xedbd:	ret = 0xeb5e; break;
	case 0xedbe:	ret = 0xf3ab; break;
	case 0xedbf:	ret = 0xeb63; break;
	case 0xedc0:	ret = 0xeb64; break;
	case 0xedc1:	ret = 0xf3b1; break;
	case 0xedc2:	ret = 0xeb6b; break;
	case 0xedc4:	ret = 0xeb6d; break;
	case 0xedc5:	ret = 0xeb6f; break;
	case 0xedc6:	ret = 0xeb77; break;
	case 0xedc7:	ret = 0xeb7d; break;
	case 0xedc8:	ret = 0xeb79; break;
	case 0xedc9:	ret = 0xeb7c; break;
	case 0xedca:	ret = 0xf3dd; break;
	case 0xedcb:	ret = 0xf3fb; break;
	case 0xedcc:	ret = 0xf3f7; break;
	case 0xedcd:	ret = 0xeb98; break;
	case 0xedce:	ret = 0xeb9a; break;
	case 0xedcf:	ret = 0xf455; break;
	case 0xedd0:	ret = 0xeba0; break;
	case 0xedd1:	ret = 0xf460; break;
	case 0xedd2:	ret = 0xebae; break;
	case 0xedd3:	ret = 0xf472; break;
	case 0xedd4:	ret = 0xebaf; break;
	case 0xedd5:	ret = 0xf479; break;
	case 0xedd7:	ret = 0xebb6; break;
	case 0xedd8:	ret = 0xebc9; break;
	case 0xedd9:	ret = 0xf4b3; break;
	case 0xedda:	ret = 0xebd0; break;
	case 0xeddb:	ret = 0xebd5; break;
	case 0xeddd:	ret = 0xf4c2; break;
	case 0xedde:	ret = 0xebdf; break;
	case 0xeddf:	ret = 0xebe9; break;
	case 0xede2:	ret = 0xebed; break;
	case 0xede3:	ret = 0xf4e6; break;
	case 0xede6:	ret = 0xebf4; break;
	case 0xede8:	ret = 0xebf8; break;
	case 0xede9:	ret = 0xf4ee; break;
	case 0xedea:	ret = 0xf4f1; break;
	case 0xedeb:	ret = 0xf543; break;
	case 0xedec:	ret = 0xec4e; break;
	case 0xeded:	ret = 0xf560; break;
	case 0xedee:	ret = 0xec5a; break;
	case 0xedef:	ret = 0xec5b; break;
	case 0xedf0:	ret = 0xf569; break;
	case 0xedf1:	ret = 0xec5d; break;
	case 0xedf2:	ret = 0xec64; break;
	case 0xedf3:	ret = 0xec66; break;
	case 0xedf4:	ret = 0xec6b; break;
	case 0xedf5:	ret = 0xf58b; break;
	case 0xedf6:	ret = 0xec76; break;
	case 0xedf7:	ret = 0xec71; break;
	case 0xedf8:	ret = 0xec72; break;
	case 0xedf9:	ret = 0xec79; break;
	case 0xedfa:	ret = 0xec7d; break;
	case 0xedfb:	ret = 0xec81; break;
	case 0xedfc:	ret = 0xf5bb; break;
	case 0xee40:	ret = 0xec89; break;
	case 0xee42:	ret = 0xec8f; break;
	case 0xee43:	ret = 0xf5d5; break;
	case 0xee45:	ret = 0xec99; break;
	case 0xee46:	ret = 0xec9b; break;
	case 0xee47:	ret = 0xec9d; break;
	case 0xee49:	ret = 0xec9f; break;
	case 0xee4b:	ret = 0xeca4; break;
	case 0xee4c:	ret = 0xeca6; break;
	case 0xee4e:	ret = 0xeca9; break;
	case 0xee4f:	ret = 0xecb1; break;
	case 0xee50:	ret = 0xecb6; break;
	case 0xee51:	ret = 0xecb9; break;
	case 0xee52:	ret = 0xecc5; break;
	case 0xee53:	ret = 0xecc8; break;
	case 0xee55:	ret = 0xf68f; break;
	case 0xee56:	ret = 0xece1; break;
	case 0xee57:	ret = 0xecdf; break;
	case 0xee58:	ret = 0xece2; break;
	case 0xee5a:	ret = 0xecf0; break;
	case 0xee5b:	ret = 0xf163; break;
	case 0xee5c:	ret = 0xf6c1; break;
	case 0xee5d:	ret = 0xed41; break;
	case 0xee5e:	ret = 0xf6c4; break;
	case 0xee60:	ret = 0xf6dc; break;
	case 0xee62:	ret = 0xed5b; break;
	case 0xee63:	ret = 0xed5c; break;
	case 0xee64:	ret = 0xf6e5; break;
	case 0xee65:	ret = 0xed60; break;
	case 0xee66:	ret = 0xf6e8; break;
	case 0xee67:	ret = 0xf758; break;
	case 0xee68:	ret = 0xf759; break;
	case 0xee6a:	ret = 0xed77; break;
	case 0xee6b:	ret = 0xed83; break;
	case 0xee6d:	ret = 0xeda0; break;
	case 0xee6e:	ret = 0xeda2; break;
	case 0xee6f:	ret = 0xf7c5; break;
	case 0xee70:	ret = 0xeda6; break;
	case 0xee71:	ret = 0xedaa; break;
	case 0xee72:	ret = 0xedb3; break;
	case 0xee73:	ret = 0xedb7; break;
	case 0xee76:	ret = 0xedea; break;
	case 0xee77:	ret = 0xedeb; break;
	case 0xee78:	ret = 0xf8b1; break;
	case 0xee79:	ret = 0xedfb; break;
	case 0xee7b:	ret = 0xee47; break;
	case 0xee7c:	ret = 0xee4e; break;
	case 0xee7d:	ret = 0xf8ef; break;
	case 0xee7e:	ret = 0xee57; break;
	case 0xee81:	ret = 0xee59; break;
	case 0xee82:	ret = 0xee5f; break;
	case 0xee83:	ret = 0xf957; break;
	case 0xee84:	ret = 0xf964; break;
	case 0xee86:	ret = 0xee8d; break;
	case 0xee87:	ret = 0xee9d; break;
	case 0xee88:	ret = 0xeea0; break;
	case 0xee89:	ret = 0xeea6; break;
	case 0xee8a:	ret = 0xeea7; break;
	case 0xee8b:	ret = 0xf9de; break;
	case 0xee8c:	ret = 0xeea9; break;
	case 0xee8d:	ret = 0xeeac; break;
	case 0xee8e:	ret = 0xeeab; break;
	case 0xee90:	ret = 0xf9ed; break;
	case 0xee91:	ret = 0xfa4f; break;
	case 0xee92:	ret = 0xeeb8; break;
	case 0xee93:	ret = 0xeebc; break;
	case 0xee97:	ret = 0xfa8e; break;
	case 0xee9a:	ret = 0xeee5; break;
	case 0xee9b:	ret = 0xeee8; break;
	case 0xee9c:	ret = 0xeeea; break;
	case 0xee9d:	ret = 0xeeee; break;
	case 0xee9e:	ret = 0xfacc; break;
	case 0xee9f:	ret = 0xeefa; break;
	case 0xeea1:	ret = 0xef41; break;
	case 0xeea2:	ret = 0xfad0; break;
	case 0xeea3:	ret = 0xeefc; break;
	case 0xeea4:	ret = 0xef40; break;
	case 0xeea6:	ret = 0xef44; break;
	case 0xeea7:	ret = 0xfad2; break;
	case 0xeea8:	ret = 0xef47; break;
	case 0xeea9:	ret = 0xef49; break;
	case 0xeeaa:	ret = 0xef48; break;
	case 0xeeab:	ret = 0xfada; break;
	case 0xeeac:	ret = 0xfadd; break;
	case 0xeead:	ret = 0xfadb; break;
	case 0xeeae:	ret = 0xef46; break;
	case 0xeeaf:	ret = 0xfae1; break;
	case 0xeeb0:	ret = 0xfaed; break;
	case 0xeeb2:	ret = 0xef4c; break;
	case 0xeeb3:	ret = 0xfafa; break;
	case 0xeeb4:	ret = 0xfaf7; break;
	case 0xeeb5:	ret = 0xfaf8; break;
	case 0xeeb6:	ret = 0xfaf6; break;
	case 0xeeb9:	ret = 0xef52; break;
	case 0xeeba:	ret = 0xef51; break;
	case 0xeebb:	ret = 0xef57; break;
	case 0xeebc:	ret = 0xef56; break;
	case 0xeebd:	ret = 0xfb43; break;
	case 0xeebf:	ret = 0xfb48; break;
	case 0xeec0:	ret = 0xef53; break;
	case 0xeec1:	ret = 0xfb47; break;
	case 0xeec2:	ret = 0xfb44; break;
	case 0xeec3:	ret = 0xef5e; break;
	case 0xeec4:	ret = 0xfb4f; break;
	case 0xeec6:	ret = 0xef62; break;
	case 0xeec7:	ret = 0xef63; break;
	case 0xeeca:	ret = 0xfb6c; break;
	case 0xeecd:	ret = 0xef7c; break;
	case 0xeecf:	ret = 0xef7d; break;
	case 0xeed0:	ret = 0xfb8c; break;
	case 0xeed1:	ret = 0xef86; break;
	case 0xeed2:	ret = 0xef87; break;
	case 0xeed4:	ret = 0xef88; break;
	case 0xeed5:	ret = 0xef8a; break;
	case 0xeed7:	ret = 0xfb9c; break;
	case 0xeed8:	ret = 0xef9f; break;
	case 0xeed9:	ret = 0xefa3; break;
	case 0xeede:	ret = 0xefa9; break;
	case 0xeedf:	ret = 0xefb1; break;
	case 0xeee1:	ret = 0xfc51; break;
	case 0xeee2:	ret = 0xefc2; break;
	case 0xeee3:	ret = 0xefc1; break;
	case 0xeee4:	ret = 0xefc5; break;
	case 0xeee5:	ret = 0xfc69; break;
	case 0xeee7:	ret = 0xefcd; break;
	case 0xeee8:	ret = 0xefdc; break;
	case 0xeee9:	ret = 0xfcb2; break;
	case 0xeeeb:	ret = 0xfccd; break;
	case 0xeeec:	ret = 0xeff0; break;
	case 0xeeef:	ret = 0x86b3; break;
	case 0xeef0:	ret = 0x86b4; break;
	case 0xeef1:	ret = 0x86b5; break;
	case 0xeef2:	ret = 0x86b6; break;
	case 0xeef3:	ret = 0x86b7; break;
	case 0xeef4:	ret = 0x86b8; break;
	case 0xeef5:	ret = 0x86b9; break;
	case 0xeef6:	ret = 0x86ba; break;
	case 0xeef7:	ret = 0x86bb; break;
	case 0xeef8:	ret = 0x86bc; break;
	case 0xeefb:	ret = 0x81ad; break;
	case 0xeefc:	ret = 0x81ae; break;
	case 0xfa40:	ret = 0x86b3; break;
	case 0xfa41:	ret = 0x86b4; break;
	case 0xfa42:	ret = 0x86b5; break;
	case 0xfa43:	ret = 0x86b6; break;
	case 0xfa44:	ret = 0x86b7; break;
	case 0xfa45:	ret = 0x86b8; break;
	case 0xfa46:	ret = 0x86b9; break;
	case 0xfa47:	ret = 0x86ba; break;
	case 0xfa48:	ret = 0x86bb; break;
	case 0xfa49:	ret = 0x86bc; break;
	case 0xfa4a:	ret = 0x8754; break;
	case 0xfa4b:	ret = 0x8755; break;
	case 0xfa4c:	ret = 0x8756; break;
	case 0xfa4d:	ret = 0x8757; break;
	case 0xfa4e:	ret = 0x8758; break;
	case 0xfa4f:	ret = 0x8759; break;
	case 0xfa50:	ret = 0x875a; break;
	case 0xfa51:	ret = 0x875b; break;
	case 0xfa52:	ret = 0x875c; break;
	case 0xfa53:	ret = 0x875d; break;
	case 0xfa56:	ret = 0x81ad; break;
	case 0xfa57:	ret = 0x81ae; break;
	case 0xfa58:	ret = 0x878a; break;
	case 0xfa59:	ret = 0x8782; break;
	case 0xfa5a:	ret = 0x8784; break;
	case 0xfa5b:	ret = 0x81e6; break;
	case 0xfa5c:	ret = 0xedb5; break;
	case 0xfa5d:	ret = 0xee8e; break;
	case 0xfa5e:	ret = 0xef58; break;
	case 0xfa5f:	ret = 0xef4d; break;
	case 0xfa60:	ret = 0xf8d3; break;
	case 0xfa61:	ret = 0x87b7; break;
	case 0xfa62:	ret = 0xf580; break;
	case 0xfa63:	ret = 0xeb54; break;
	case 0xfa64:	ret = 0xeb89; break;
	case 0xfa65:	ret = 0xfb42; break;
	case 0xfa66:	ret = 0xeb56; break;
	case 0xfa67:	ret = 0xeab8; break;
	case 0xfa68:	ret = 0x87a2; break;
	case 0xfa69:	ret = 0x87a6; break;
	case 0xfa6b:	ret = 0xf05e; break;
	case 0xfa6c:	ret = 0x87a9; break;
	case 0xfa6e:	ret = 0x87ae; break;
	case 0xfa6f:	ret = 0xf06c; break;
	case 0xfa70:	ret = 0x87b3; break;
	case 0xfa71:	ret = 0xf06d; break;
	case 0xfa72:	ret = 0x87b4; break;
	case 0xfa74:	ret = 0x87bf; break;
	case 0xfa75:	ret = 0xf07c; break;
	case 0xfa77:	ret = 0xf07b; break;
	case 0xfa78:	ret = 0xf080; break;
	case 0xfa79:	ret = 0xf087; break;
	case 0xfa7a:	ret = 0xf07e; break;
	case 0xfa7b:	ret = 0x87c2; break;
	case 0xfa7e:	ret = 0x87cd; break;
	case 0xfa81:	ret = 0x87d2; break;
	case 0xfa84:	ret = 0x87db; break;
	case 0xfa86:	ret = 0xf162; break;
	case 0xfa88:	ret = 0x87e5; break;
	case 0xfa89:	ret = 0x87e9; break;
	case 0xfa8a:	ret = 0x87ea; break;
	case 0xfa8b:	ret = 0xf16d; break;
	case 0xfa8d:	ret = 0x87f0; break;
	case 0xfa8e:	ret = 0x87f2; break;
	case 0xfa91:	ret = 0x87f6; break;
	case 0xfa92:	ret = 0xf18c; break;
	case 0xfa93:	ret = 0x8840; break;
	case 0xfa94:	ret = 0x8842; break;
	case 0xfa95:	ret = 0x8849; break;
	case 0xfa98:	ret = 0xf1ea; break;
	case 0xfa99:	ret = 0x886e; break;
	case 0xfa9a:	ret = 0x886d; break;
	case 0xfa9b:	ret = 0x886a; break;
	case 0xfa9c:	ret = 0x8876; break;
	case 0xfa9d:	ret = 0x887c; break;
	case 0xfa9f:	ret = 0xf259; break;
	case 0xfaa1:	ret = 0xf264; break;
	case 0xfaa2:	ret = 0x8889; break;
	case 0xfaa4:	ret = 0x888d; break;
	case 0xfaa6:	ret = 0x9875; break;
	case 0xfaa7:	ret = 0xf0a2; break;
	case 0xfaa8:	ret = 0xecc7; break;
	case 0xfaa9:	ret = 0x9878; break;
	case 0xfaaa:	ret = 0x9879; break;
	case 0xfaab:	ret = 0x987b; break;
	case 0xfaac:	ret = 0xf0c1; break;
	case 0xfaad:	ret = 0x9889; break;
	case 0xfaaf:	ret = 0x9891; break;
	case 0xfab0:	ret = 0x9895; break;
	case 0xfab1:	ret = 0x9892; break;
	case 0xfab2:	ret = 0xf0d3; break;
	case 0xfab3:	ret = 0x9897; break;
	case 0xfab4:	ret = 0x989c; break;
	case 0xfab5:	ret = 0xf0e5; break;
	case 0xfab8:	ret = 0xeab4; break;
	case 0xfab9:	ret = 0xeabc; break;
	case 0xfaba:	ret = 0xeac3; break;
	case 0xfabb:	ret = 0xeac5; break;
	case 0xfabe:	ret = 0xf2cb; break;
	case 0xfac0:	ret = 0xead3; break;
	case 0xfac2:	ret = 0xf2d6; break;
	case 0xfac4:	ret = 0xead9; break;
	case 0xfac5:	ret = 0xead8; break;
	case 0xfac6:	ret = 0xf2e7; break;
	case 0xfac8:	ret = 0xf341; break;
	case 0xfac9:	ret = 0xeaf2; break;
	case 0xfaca:	ret = 0xeaf8; break;
	case 0xfacb:	ret = 0xf36f; break;
	case 0xfacc:	ret = 0xeafa; break;
	case 0xface:	ret = 0xeb4b; break;
	case 0xfacf:	ret = 0xeb4d; break;
	case 0xfad1:	ret = 0xeb4c; break;
	case 0xfad3:	ret = 0xeb4e; break;
	case 0xfad4:	ret = 0xeb51; break;
	case 0xfad5:	ret = 0xeb5d; break;
	case 0xfad6:	ret = 0xf39e; break;
	case 0xfad7:	ret = 0xeb5a; break;
	case 0xfad9:	ret = 0xeb5e; break;
	case 0xfada:	ret = 0xf3ab; break;
	case 0xfadb:	ret = 0xeb63; break;
	case 0xfadc:	ret = 0xeb64; break;
	case 0xfadd:	ret = 0xf3b1; break;
	case 0xfade:	ret = 0xeb6b; break;
	case 0xfae0:	ret = 0xeb6d; break;
	case 0xfae1:	ret = 0xeb6f; break;
	case 0xfae2:	ret = 0xeb77; break;
	case 0xfae3:	ret = 0xeb7d; break;
	case 0xfae4:	ret = 0xeb79; break;
	case 0xfae5:	ret = 0xeb7c; break;
	case 0xfae6:	ret = 0xf3dd; break;
	case 0xfae7:	ret = 0xf3fb; break;
	case 0xfae8:	ret = 0xf3f7; break;
	case 0xfae9:	ret = 0xeb98; break;
	case 0xfaea:	ret = 0xeb9a; break;
	case 0xfaeb:	ret = 0xf455; break;
	case 0xfaec:	ret = 0xeba0; break;
	case 0xfaed:	ret = 0xf460; break;
	case 0xfaee:	ret = 0xebae; break;
	case 0xfaef:	ret = 0xf472; break;
	case 0xfaf0:	ret = 0xebaf; break;
	case 0xfaf1:	ret = 0xf479; break;
	case 0xfaf3:	ret = 0xebb6; break;
	case 0xfaf4:	ret = 0xebc9; break;
	case 0xfaf5:	ret = 0xf4b3; break;
	case 0xfaf6:	ret = 0xebd0; break;
	case 0xfaf7:	ret = 0xebd5; break;
	case 0xfaf9:	ret = 0xf4c2; break;
	case 0xfafa:	ret = 0xebdf; break;
	case 0xfafb:	ret = 0xebe9; break;
	case 0xfb41:	ret = 0xebed; break;
	case 0xfb42:	ret = 0xf4e6; break;
	case 0xfb45:	ret = 0xebf4; break;
	case 0xfb47:	ret = 0xebf8; break;
	case 0xfb48:	ret = 0xf4ee; break;
	case 0xfb49:	ret = 0xf4f1; break;
	case 0xfb4a:	ret = 0xf543; break;
	case 0xfb4b:	ret = 0xec4e; break;
	case 0xfb4c:	ret = 0xf560; break;
	case 0xfb4d:	ret = 0xec5a; break;
	case 0xfb4e:	ret = 0xec5b; break;
	case 0xfb4f:	ret = 0xf569; break;
	case 0xfb50:	ret = 0xec5d; break;
	case 0xfb51:	ret = 0xec64; break;
	case 0xfb52:	ret = 0xec66; break;
	case 0xfb53:	ret = 0xec6b; break;
	case 0xfb54:	ret = 0xf58b; break;
	case 0xfb55:	ret = 0xec76; break;
	case 0xfb56:	ret = 0xec71; break;
	case 0xfb57:	ret = 0xec72; break;
	case 0xfb58:	ret = 0xec79; break;
	case 0xfb59:	ret = 0xec7d; break;
	case 0xfb5a:	ret = 0xec81; break;
	case 0xfb5b:	ret = 0xf5bb; break;
	case 0xfb5c:	ret = 0xec89; break;
	case 0xfb5e:	ret = 0xec8f; break;
	case 0xfb5f:	ret = 0xf5d5; break;
	case 0xfb61:	ret = 0xec99; break;
	case 0xfb62:	ret = 0xec9b; break;
	case 0xfb63:	ret = 0xec9d; break;
	case 0xfb65:	ret = 0xec9f; break;
	case 0xfb67:	ret = 0xeca4; break;
	case 0xfb68:	ret = 0xeca6; break;
	case 0xfb6a:	ret = 0xeca9; break;
	case 0xfb6b:	ret = 0xecb1; break;
	case 0xfb6c:	ret = 0xecb6; break;
	case 0xfb6d:	ret = 0xecb9; break;
	case 0xfb6e:	ret = 0xecc5; break;
	case 0xfb6f:	ret = 0xecc8; break;
	case 0xfb71:	ret = 0xf68f; break;
	case 0xfb72:	ret = 0xece1; break;
	case 0xfb73:	ret = 0xecdf; break;
	case 0xfb74:	ret = 0xece2; break;
	case 0xfb76:	ret = 0xecf0; break;
	case 0xfb77:	ret = 0xf163; break;
	case 0xfb78:	ret = 0xf6c1; break;
	case 0xfb79:	ret = 0xed41; break;
	case 0xfb7a:	ret = 0xf6c4; break;
	case 0xfb7c:	ret = 0xf6dc; break;
	case 0xfb7e:	ret = 0xed5b; break;
	case 0xfb80:	ret = 0xed5c; break;
	case 0xfb81:	ret = 0xf6e5; break;
	case 0xfb82:	ret = 0xed60; break;
	case 0xfb83:	ret = 0xf6e8; break;
	case 0xfb84:	ret = 0xf758; break;
	case 0xfb85:	ret = 0xf759; break;
	case 0xfb87:	ret = 0xed77; break;
	case 0xfb88:	ret = 0xed83; break;
	case 0xfb8a:	ret = 0xeda0; break;
	case 0xfb8b:	ret = 0xeda2; break;
	case 0xfb8c:	ret = 0xf7c5; break;
	case 0xfb8d:	ret = 0xeda6; break;
	case 0xfb8e:	ret = 0xedaa; break;
	case 0xfb8f:	ret = 0xedb3; break;
	case 0xfb90:	ret = 0xedb7; break;
	case 0xfb93:	ret = 0xedea; break;
	case 0xfb94:	ret = 0xedeb; break;
	case 0xfb95:	ret = 0xf8b1; break;
	case 0xfb96:	ret = 0xedfb; break;
	case 0xfb98:	ret = 0xee47; break;
	case 0xfb99:	ret = 0xee4e; break;
	case 0xfb9a:	ret = 0xf8ef; break;
	case 0xfb9b:	ret = 0xee57; break;
	case 0xfb9d:	ret = 0xee59; break;
	case 0xfb9e:	ret = 0xee5f; break;
	case 0xfb9f:	ret = 0xf957; break;
	case 0xfba0:	ret = 0xf964; break;
	case 0xfba2:	ret = 0xee8d; break;
	case 0xfba3:	ret = 0xee9d; break;
	case 0xfba4:	ret = 0xeea0; break;
	case 0xfba5:	ret = 0xeea6; break;
	case 0xfba6:	ret = 0xeea7; break;
	case 0xfba7:	ret = 0xf9de; break;
	case 0xfba8:	ret = 0xeea9; break;
	case 0xfba9:	ret = 0xeeac; break;
	case 0xfbaa:	ret = 0xeeab; break;
	case 0xfbac:	ret = 0xf9ed; break;
	case 0xfbad:	ret = 0xfa4f; break;
	case 0xfbae:	ret = 0xeeb8; break;
	case 0xfbaf:	ret = 0xeebc; break;
	case 0xfbb3:	ret = 0xfa8e; break;
	case 0xfbb6:	ret = 0xeee5; break;
	case 0xfbb7:	ret = 0xeee8; break;
	case 0xfbb8:	ret = 0xeeea; break;
	case 0xfbb9:	ret = 0xeeee; break;
	case 0xfbba:	ret = 0xfacc; break;
	case 0xfbbb:	ret = 0xeefa; break;
	case 0xfbbd:	ret = 0xef41; break;
	case 0xfbbe:	ret = 0xfad0; break;
	case 0xfbbf:	ret = 0xeefc; break;
	case 0xfbc0:	ret = 0xef40; break;
	case 0xfbc2:	ret = 0xef44; break;
	case 0xfbc3:	ret = 0xfad2; break;
	case 0xfbc4:	ret = 0xef47; break;
	case 0xfbc5:	ret = 0xef49; break;
	case 0xfbc6:	ret = 0xef48; break;
	case 0xfbc7:	ret = 0xfada; break;
	case 0xfbc8:	ret = 0xfadd; break;
	case 0xfbc9:	ret = 0xfadb; break;
	case 0xfbca:	ret = 0xef46; break;
	case 0xfbcb:	ret = 0xfae1; break;
	case 0xfbcc:	ret = 0xfaed; break;
	case 0xfbce:	ret = 0xef4c; break;
	case 0xfbcf:	ret = 0xfafa; break;
	case 0xfbd0:	ret = 0xfaf7; break;
	case 0xfbd1:	ret = 0xfaf8; break;
	case 0xfbd2:	ret = 0xfaf6; break;
	case 0xfbd5:	ret = 0xef52; break;
	case 0xfbd6:	ret = 0xef51; break;
	case 0xfbd7:	ret = 0xef57; break;
	case 0xfbd8:	ret = 0xef56; break;
	case 0xfbd9:	ret = 0xfb43; break;
	case 0xfbdb:	ret = 0xfb48; break;
	case 0xfbdc:	ret = 0xef53; break;
	case 0xfbdd:	ret = 0xfb47; break;
	case 0xfbde:	ret = 0xfb44; break;
	case 0xfbdf:	ret = 0xef5e; break;
	case 0xfbe0:	ret = 0xfb4f; break;
	case 0xfbe2:	ret = 0xef62; break;
	case 0xfbe3:	ret = 0xef63; break;
	case 0xfbe6:	ret = 0xfb6c; break;
	case 0xfbe9:	ret = 0xef7c; break;
	case 0xfbeb:	ret = 0xef7d; break;
	case 0xfbec:	ret = 0xfb8c; break;
	case 0xfbed:	ret = 0xef86; break;
	case 0xfbee:	ret = 0xef87; break;
	case 0xfbf0:	ret = 0xef88; break;
	case 0xfbf1:	ret = 0xef8a; break;
	case 0xfbf3:	ret = 0xfb9c; break;
	case 0xfbf4:	ret = 0xef9f; break;
	case 0xfbf5:	ret = 0xefa3; break;
	case 0xfbfa:	ret = 0xefa9; break;
	case 0xfbfb:	ret = 0xefb1; break;
	case 0xfc40:	ret = 0xfc51; break;
	case 0xfc41:	ret = 0xefc2; break;
	case 0xfc42:	ret = 0xefc1; break;
	case 0xfc43:	ret = 0xefc5; break;
	case 0xfc44:	ret = 0xfc69; break;
	case 0xfc46:	ret = 0xefcd; break;
	case 0xfc47:	ret = 0xefdc; break;
	case 0xfc48:	ret = 0xfcb2; break;
	case 0xfc4a:	ret = 0xfccd; break;
	case 0xfc4b:	ret = 0xeff0; break;
	default:	ret = code;   break;
	}
	
	return ret;
}
