static int sdl_keytable[SDLK_LAST] = {
	0, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_BACKSPACE, KEY_TAB, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_CLEAR, KEY_RETURN, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_PAUSE, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_ESCAPE, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_SPACE, 33, 34, 35, 
	36, KEY_UNDEFINED, 38, 39, 
	40, 41, 42, 43, 
	44, 45, 46, 47, 
	KEY_0, KEY_1, KEY_2, KEY_3, 
	KEY_4, KEY_5, KEY_6, KEY_7, 
	KEY_8, KEY_9, 58, 59, 
	60, 61, 62, 63, 
	64, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 91, 
	92, 93, 94, 95, 
	96, KEY_A, KEY_B, KEY_C, 
	KEY_D, KEY_E, KEY_F, KEY_G, 
	KEY_H, KEY_I, KEY_J, KEY_K, 
	KEY_L, KEY_M, KEY_N, KEY_O, 
	KEY_P, KEY_Q, KEY_R, KEY_S, 
	KEY_T, KEY_U, KEY_V, KEY_W, 
	KEY_X, KEY_Y, KEY_Z, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_DELETE, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	0 /* SDL_WORLD_0 */, 0 /* SDL_WORLD_1 */, 0 /* SDL_WORLD_2 */, 0 /* SDL_WORLD_3 */, 
	0 /* SDL_WORLD_4 */, 0 /* SDL_WORLD_5 */, 0 /* SDL_WORLD_6 */, 0 /* SDL_WORLD_7 */, 
	0 /* SDL_WORLD_8 */, 0 /* SDL_WORLD_9 */, 0 /* SDL_WORLD_10 */, 0 /* SDL_WORLD_11 */, 
	0 /* SDL_WORLD_12 */, 0 /* SDL_WORLD_13 */, 0 /* SDL_WORLD_14 */, 0 /* SDL_WORLD_15 */, 
	0 /* SDL_WORLD_16 */, 0 /* SDL_WORLD_17 */, 0 /* SDL_WORLD_18 */, 0 /* SDL_WORLD_19 */, 
	0 /* SDL_WORLD_20 */, 0 /* SDL_WORLD_21 */, 0 /* SDL_WORLD_22 */, 0 /* SDL_WORLD_23 */, 
	0 /* SDL_WORLD_24 */, 0 /* SDL_WORLD_25 */, 0 /* SDL_WORLD_26 */, 0 /* SDL_WORLD_27 */, 
	0 /* SDL_WORLD_28 */, 0 /* SDL_WORLD_29 */, 0 /* SDL_WORLD_30 */, 0 /* SDL_WORLD_31 */, 
	0 /* SDL_WORLD_32 */, 0 /* SDL_WORLD_33 */, 0 /* SDL_WORLD_34 */, 0 /* SDL_WORLD_35 */, 
	0 /* SDL_WORLD_36 */, 0 /* SDL_WORLD_37 */, 0 /* SDL_WORLD_38 */, 0 /* SDL_WORLD_39 */, 
	0 /* SDL_WORLD_40 */, 0 /* SDL_WORLD_41 */, 0 /* SDL_WORLD_42 */, 0 /* SDL_WORLD_43 */, 
	0 /* SDL_WORLD_44 */, 0 /* SDL_WORLD_45 */, 0 /* SDL_WORLD_46 */, 0 /* SDL_WORLD_47 */, 
	0 /* SDL_WORLD_48 */, 0 /* SDL_WORLD_49 */, 0 /* SDL_WORLD_50 */, 0 /* SDL_WORLD_51 */, 
	0 /* SDL_WORLD_52 */, 0 /* SDL_WORLD_53 */, 0 /* SDL_WORLD_54 */, 0 /* SDL_WORLD_55 */, 
	0 /* SDL_WORLD_56 */, 0 /* SDL_WORLD_57 */, 0 /* SDL_WORLD_58 */, 0 /* SDL_WORLD_59 */, 
	0 /* SDL_WORLD_60 */, 0 /* SDL_WORLD_61 */, 0 /* SDL_WORLD_62 */, 0 /* SDL_WORLD_63 */, 
	0 /* SDL_WORLD_64 */, 0 /* SDL_WORLD_65 */, 0 /* SDL_WORLD_66 */, 0 /* SDL_WORLD_67 */, 
	0 /* SDL_WORLD_68 */, 0 /* SDL_WORLD_69 */, 0 /* SDL_WORLD_70 */, 0 /* SDL_WORLD_71 */, 
	0 /* SDL_WORLD_72 */, 0 /* SDL_WORLD_73 */, 0 /* SDL_WORLD_74 */, 0 /* SDL_WORLD_75 */, 
	0 /* SDL_WORLD_76 */, 0 /* SDL_WORLD_77 */, 0 /* SDL_WORLD_78 */, 0 /* SDL_WORLD_79 */, 
	0 /* SDL_WORLD_80 */, 0 /* SDL_WORLD_81 */, 0 /* SDL_WORLD_82 */, 0 /* SDL_WORLD_83 */, 
	0 /* SDL_WORLD_84 */, 0 /* SDL_WORLD_85 */, 0 /* SDL_WORLD_86 */, 0 /* SDL_WORLD_87 */, 
	0 /* SDL_WORLD_88 */, 0 /* SDL_WORLD_89 */, 0 /* SDL_WORLD_90 */, 0 /* SDL_WORLD_91 */, 
	0 /* SDL_WORLD_92 */, 0 /* SDL_WORLD_93 */, 0 /* SDL_WORLD_94 */, 0 /* SDL_WORLD_95 */, 
	KEY_PAD_0, KEY_PAD_1, KEY_PAD_2, KEY_PAD_3, 
	KEY_PAD_4, KEY_PAD_5, KEY_PAD_6, KEY_PAD_7, 
	KEY_PAD_8, KEY_PAD_9, KEY_PAD_PERIOD, KEY_PAD_DIVIDE, 
	KEY_PAD_MULTIPLY, KEY_PAD_MINUS, KEY_PAD_PLUS, KEY_PAD_ENTER, 
	KEY_PAD_EQUALS, KEY_UP, KEY_DOWN, KEY_RIGHT, 
	KEY_LEFT, KEY_INSERT, KEY_HOME, KEY_END, 
	KEY_PAGEUP, KEY_PAGEDOWN, KEY_F1, KEY_F2, 
	KEY_F3, KEY_F4, KEY_F5, KEY_F6, 
	KEY_F7, KEY_F8, KEY_F9, KEY_F10, 
	KEY_F11, KEY_F12, KEY_F13, KEY_F14, 
	KEY_F15, KEY_UNDEFINED, KEY_UNDEFINED, KEY_UNDEFINED, 
	KEY_NUMLOCK, KEY_CAPSLOCK, KEY_SCROLLOCK, KEY_RSHIFT, 
	KEY_RSHIFT, KEY_RCTRL, KEY_RCTRL, KEY_RALT, 
	KEY_RALT, 0 /* SDL_RMETA */, 0 /* SDL_LMETA */, 0 /* SDL_LSUPER */, 
	0 /* SDL_RSUPER */, 0 /* SDL_MODE */, KEY_UNDEFINED, KEY_HELP, 
	0 /* SDL_PRINT */, 0 /* SDL_SYSREQ */, 0 /* SDL_BREAK */, 0 /* SDL_MENU */, 
	0 /* SDL_POWER */, 0 /* SDL_EURO */, };
